#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include "express.h"

#define ERRMSG(a)  ERRverror("ParssimParser",ERR_PRINT,a)
#define oops(msg) { perror(msg); return(0); }

int genOutput(OMobj_id, char *, char *, char *, int, int);
int getTimes(char *, int *, int *, int *, int *);

int
PLTparser(OMobj_id ParssimParser_id, OMevent_mask event_mask, int seq_num)
{
  /***********************/
  /*  Declare variables  */
  /***********************/
  int run;
  double trigger;
  int partial, k, rc;
  FILE *fp_in;
  char buf[64];
  char lbuf[32];
  char pbuf[32];
  char dbuf[32];
  char *dataDir = NULL;
  char *inputDeck = NULL;
  char scratch[2048];
  char inTag[32] = "$S CHEMICAL SPECIES\n";
  int nComps, nProducts, nSpecies;
  int prevStep, prevTime, curStep, curTime;
  char step[4];
  char *strStep;
  char pv[16] = "vel.";
  char pp[16] = "pres.";
  char pc[32] = "conc";
  int quotient, remainder;
  int i, offset;
  int selConc;
  int concIndex = -1;
  int ilrange;
  int iurange;
  char lrange[8];
  char urange[8];
  char range[16];
  char inFile[64];
  char *inPLT = NULL;
  char rootname[64];
  char *p = NULL;
  char *token = NULL;
  char titleLine[32];
  char *title = NULL;
  char pointsFile[64];
  char dataFile[64];
  char *timeFile = NULL;
  int tmp1, tmp2;
  /*
  char *projDir = NULL;
  */

  /*************************************************************/
  /*  dataDir, inputDeck and run need to be set before reading */
  /*************************************************************/
  /* Get dataDir's value */
  if (OMget_name_str_val(ParssimParser_id, OMstr_to_name("dataDir"), 
      &dataDir, 0) != 1) {
    ERRMSG("OMget failed for dataDir\n");
    dataDir = NULL;
    return(0);
  }
  /* Get inputDeck's value */
  if (OMget_name_str_val(ParssimParser_id, OMstr_to_name("inputDeck"), 
      &inputDeck, 0) != 1) {
    ERRMSG("OMget failed for inputDeck\n");
    inputDeck = NULL;
    return(0);
  }
  /* Get run's value */
  if (OMget_name_int_val(ParssimParser_id, OMstr_to_name("run"), 
	&run) != 1) {
    ERRMSG("OMget failed for run\n");
    run = 0;
    return(0);
  }
  /* Get selConc's value */
  if (OMget_name_int_val(ParssimParser_id, OMstr_to_name("selConc"), 
	&selConc) != 1) {
    ERRMSG("OMget failed for selConc\n");
    selConc = 1;
    return(0);
  }

  /* Form input deck filename relative to project directory */
  strcpy(buf, dataDir);
  strcat(buf, "/");
  strcat(buf, inputDeck);

  /* Open input deck to extract number of chemical species */
  fp_in = fopen(buf, "r");
  while( (fgets(scratch, sizeof(scratch), fp_in)) != NULL ) {
    if(!strcmp(scratch, inTag)) {
      fscanf(fp_in, " %d", &nComps);
      fgets(scratch, sizeof(scratch), fp_in);
      fscanf(fp_in, " %d", &nProducts);
    }
  }
  fclose(fp_in);
  
  nSpecies = nComps + nProducts;

  /* Get value of project directory if possible
  projDir = getenv("XP_PATH");

  if (projDir == NULL) {

    oops("\nPLTparser: Can't get value of projDir\n");

  }

  strtok(projDir, " ");
  */

  /*********************/
  /* The run = 1 block */
  /*********************/

  if (run == 1) {

    /* Get trigger's value */
    if (OMget_name_real_val(ParssimParser_id, OMstr_to_name("trigger"),
        &trigger) != 1) {
      ERRMSG("OMget failed for trigger\n");
      trigger= 0.0;
      return(0);
    }

    /* Begin processing *Time files to determine appropriate */
    /* .plt files to open */

    /*
    strcpy(buf, projDir);
    strcat(buf, "/data/velTime");
    */

    strcpy(buf, dataDir);
    strcat(buf, "/velTime");

    /* Velocities */
    rc = getTimes(buf, &curStep, &curTime, &prevStep, &prevTime); 
    if (rc == 1) {
      if (curStep != 0) {
	  strcpy(buf, dataDir);
        strcat(buf, "/");
        strcat(buf, pv);
        sprintf(step, "%.3d", prevStep);
        strcat(buf, step);
        strcpy(pbuf, buf);
        strcpy(dbuf, buf);
        strcat(buf, ".plt");
        strcpy(inFile, buf);
        strcat(pbuf, ".pts");
        strcpy(pointsFile, pbuf);
        strcat(dbuf, ".out");
        strcpy(dataFile, dbuf);
        genOutput(ParssimParser_id, inFile, pointsFile,
		      dataFile, concIndex, prevTime);
      }
    }
    
    /*
    strcpy(buf, projDir);
    strcat(buf, "/data/presTime");
    */

    strcpy(buf, dataDir);
    strcat(buf, "/presTime");

    /* Pressures */
    rc = getTimes(buf, &curStep, &curTime, &prevStep, &prevTime);
    if (rc == 1) {
      if (curStep != 0) {
	  strcpy(buf, dataDir);
        strcat(buf, "/");
        strcat(buf, pp);
        sprintf(step, "%.3d", prevStep);
        strcat(buf, step);
        strcpy(pbuf, buf);
        strcpy(dbuf, buf);
        strcat(buf, ".plt");
        strcpy(inFile, buf);
        strcat(pbuf, ".pts");
        strcpy(pointsFile, pbuf);
        strcat(dbuf, ".out");
        strcpy(dataFile, dbuf);
        genOutput(ParssimParser_id, inFile, pointsFile,
                  dataFile, concIndex, prevTime);
      }
    }
    /* Concentrations */

    /* 
    strcpy(buf, projDir);
    strcat(buf, "/data/concTime");
    */

    strcpy(buf, dataDir);
    strcat(buf, "/concTime");

    rc = getTimes(buf, &curStep, &curTime, &prevStep, &prevTime);
    if (rc == 1) {
	/* somewhat ad hoc "50" here...need to fortify with */
	/* information from input deck */
	curStep = curStep / 50;
      if (curStep != 0) {
        /* In order to open concentration filenames, we */
        /* need to form variable concentration filename strings */
        remainder = nSpecies % 10;
        if (remainder != 0) 
          partial = 1;
        else
          partial = 0;
        quotient = nSpecies / 10;
        for(i = 1; i <= quotient + 1; i++) {
          offset = 10 * (i - 1);
          sprintf(lrange, "%d", offset + 1);
	    ilrange = offset + 1;
	    strcpy(lbuf, lrange);
          strcat(lbuf, "-");
          if(i == (quotient + 1)) {
            sprintf(urange, "%d", offset + remainder);
		iurange = offset + remainder;
	    } else {
            sprintf(urange, "%d", offset + 10);
		iurange = offset + 10;
	    }
          strcat(lbuf, urange);
          strcpy(range, lbuf);
          if ((selConc >= ilrange) && (selConc <= iurange)) {
	      strcpy(buf, dataDir);
	      strcat(buf, "/");
            strcat(buf, pc);
	      strcat(buf, range);
	      strcat(buf, ".");
            sprintf(step, "%.3d", prevStep/50);
            strcat(buf, step);
            strcpy(pbuf, buf);
            strcpy(dbuf, buf);
            strcat(buf, ".plt");
            strcpy(inFile, buf);
            strcat(pbuf, ".pts");
            strcpy(pointsFile, pbuf);
            strcat(dbuf, ".out");
            strcpy(dataFile, dbuf);
	      concIndex = selConc - offset;
            genOutput(ParssimParser_id, inFile, pointsFile,
                      dataFile, concIndex, prevTime);
	    }
        }
      }
    }

  /*********************/
  /* The run = 0 block */
  /*********************/

  } else if (run == 0) {
      
    /* Get inPLT's value */
    if (OMget_name_str_val(ParssimParser_id, OMstr_to_name("inPLT"),
        &inPLT, 0) != 1) {
      ERRMSG("OMget failed for inPLT\n");
      inPLT = NULL;
      return(0);
    }
    
    fprintf(stderr, "\nrun = 0: inPLT is %s\n", inPLT);

    strcpy(rootname, inPLT);
    p = strrchr(rootname, '.');
    *p = '\0';
    strcat(rootname, ".pts");
    strcpy(pointsFile, rootname);
    p = strrchr(rootname, '.');
    *p = '\0';
    strcat(rootname, ".out");
    strcpy(dataFile, rootname);

    concIndex = selConc % 10;

    
    fp_in = fopen(inPLT, "r");
    fgets(titleLine, sizeof(titleLine), fp_in);
    token = strtok(titleLine, "\"");
    title = strtok(NULL, "\""); 
    fclose(fp_in);

    strcpy(buf, inPLT);
    token = strtok(buf, ".");
    strStep = strtok(NULL, ".");

    curStep = atoi(strStep);

    if (!strcmp(title, "velocity")) {
	strcpy(buf, dataDir);
      strcat(buf, "/velTime");
	strcpy(timeFile, buf);
    } else if (!strcmp(title, "pressure")) {
	strcpy(buf, dataDir);
      strcat(buf, "/presTime");
	strcpy(timeFile, buf);
    } else if (!strcmp(title, "concentration")) {
	strcpy(buf, dataDir);
      strcat(buf, "/concTime");
	strcpy(timeFile, buf);
	curStep = curStep * 50;
    } else {
      oops("\nPLTparser: Couldn't determine timeFile\n");
    }

    fp_in = fopen(timeFile, "r");
    while((fscanf(fp_in, "%d %d", &tmp1, &tmp2)) != EOF) { 
      if (tmp1 == curStep) {
        curTime = tmp2;
 	  break;
	}
    }
    fclose(fp_in);

    genOutput(ParssimParser_id, inPLT, pointsFile,
              dataFile, concIndex, curTime);
    

  } else {
	
	return (0);

  }

  return (1);

}

int genOutput(OMobj_id ParssimParser_id, 
		  char *inFile, char *pointsFile, char *dataFile,
 		  int concIndex, int prevTime)
{

  FILE *fp_in;
  FILE *fp_points;
  FILE *fp_vel;
  FILE *fp_pres;
  FILE *fp_conc;
  char velFile[32];
  char presFile[32];
  char concFile[32];
  char *token = NULL;
  char titleLine[32];
  char *title = NULL;
  char varNamesLine[512];
  char *varNames[256];
  char *dataNames[256];
  char scratch[1024];
  int velTime, presTime, concTime;
  int i;
  int dim1, dim2, dim3;
  int prod_dims;
  float *conc = NULL;
  float *coords = NULL;
  float *vels = NULL;
  float *pressures = NULL;

  fprintf(stderr, "Generating some output files...\n");

  if (inFile) {

    fp_in = fopen(inFile, "r");

    if (!fp_in) {
      oops("\ngenOutput: Can't open inFile\n");
    }

  } else {
        
     oops("\ngenOutput: Can't get value of inFile\n");

  }

  /* Extract TITLE */
  fgets(titleLine, sizeof(titleLine), fp_in);
  token = strtok(titleLine, "\"");
  title = strtok(NULL, "\"");

  /* Extract VARIABLES */
  fgets(varNamesLine, sizeof(varNamesLine), fp_in);

  fgets(scratch, sizeof(scratch), fp_in);
  fscanf(fp_in, " I=%d, J=%d, K=%d", &dim1, &dim2, &dim3);
  prod_dims = dim1 * dim2 * dim3;
  fgets(scratch, sizeof(scratch), fp_in);
  fgets(scratch, sizeof(scratch), fp_in);

  if (!strcmp(title, "concentration")) {

    concTime = prevTime;
    OMset_name_int_val(ParssimParser_id, OMstr_to_name("concTime"), 
			     concTime);

    strcpy(concFile, dataFile);

    if (pointsFile != NULL) {

      fp_points = fopen(pointsFile, "w");

      if (fp_points == NULL) {
        oops("\ngenOutput: Can't open pointsFile\n");
      }

    } else {

       oops("\ngenOutput: Can't get value of pointsFile\n");

    }

    if (concFile) {

      fp_conc = fopen(concFile, "w");

      if (!fp_conc) {
        oops("\ngenOutput: Can't open concFile\n");
      }

    } else {

       oops("\ngenOutput: Can't get value of concFile\n");

    }

    coords = (float *) malloc(3 * prod_dims * sizeof(float));	

    fprintf(fp_points, "%d %d %d\n", dim1, dim2, dim3);
    fprintf(fp_conc, "%d %d %d\n", dim1, dim2, dim3);
    conc = (float *) malloc(prod_dims * sizeof(float));
    for (i = 0; i < prod_dims; i++) {
      fscanf(fp_in, " %f %f %f ",
             &coords[3*i], &coords[3*i+1], &coords[3*i+2]);
      switch(concIndex) {
        case 10:
          fscanf(fp_in, " %*e %*e %*e %*e %*e %*e %*e %*e %*e %e ",
                 &conc[i]);
	    break;
        case 9:
          fscanf(fp_in, " %*e %*e %*e %*e %*e %*e %*e %*e %e %*e ",
                 &conc[i]);
	    break;
        case 8:
          fscanf(fp_in, " %*e %*e %*e %*e %*e %*e %*e %e %*e %*e ",
                 &conc[i]);
	    break;
        case 7:
          fscanf(fp_in, " %*e %*e %*e %*e %*e %*e %e %*e %*e %*e ",
                 &conc[i]);
	    break;
        case 6:
          fscanf(fp_in, " %*e %*e %*e %*e %*e %e %*e %*e %*e %*e ",
                 &conc[i]);
	    break;
        case 5:
          fscanf(fp_in, " %*e %*e %*e %*e %e %*e %*e %*e %*e %*e ",
                 &conc[i]);
	    break;
        case 4:
          fscanf(fp_in, " %*e %*e %*e %e %*e %*e %*e %*e %*e %*e ",
                 &conc[i]);
	    break;
        case 3:
          fscanf(fp_in, " %*e %*e %e %*e %*e %*e %*e %*e %*e %*e ",
                 &conc[i]);
	    break;
        case 2:
          fscanf(fp_in, " %*e %e %*e %*e %*e %*e %*e %*e %*e %*e ",
                 &conc[i]);
	    break;
        case 1:
          fscanf(fp_in, " %e %*e %*e %*e %*e %*e %*e %*e %*e %*e ",
                 &conc[i]);
	    break;
	  default:
          oops("\ngenOutput: Couldn't create concentration array\n");
	    break;
      }

      fprintf(fp_conc, "%e\n", conc[i]);

    }

    /* Set concFile's value */
    OMset_name_str_val(ParssimParser_id, OMstr_to_name("concFile"),
                             concFile);

    for (i = 0; i < dim1; i++)
       fprintf(fp_points, "%f 0 0\n", coords[3*i]);
    for (i = 0; i < dim2; i++)
       fprintf(fp_points, "0 %f 0\n", coords[(3*i)*dim1 + 1]);
    for (i = 0; i < dim3; i++)
       fprintf(fp_points, "0 0 %f\n", coords[3*i*dim1*dim2 + 2]);


    /* Set pointsFile's value */
    OMset_name_str_val(ParssimParser_id, 
    			     OMstr_to_name("pointsFile"), pointsFile);
    free(coords);
    free(conc);
    fclose(fp_conc);
    fclose(fp_points);

  } else if (!strcmp(title, "velocity")) {

    velTime = prevTime;
    OMset_name_int_val(ParssimParser_id, OMstr_to_name("velTime"),
                       velTime);

    strcpy(velFile, dataFile);
    if (pointsFile != NULL) {

      fp_points = fopen(pointsFile, "w");

      if (fp_points == NULL) {
        oops("\ngenOutput: Can't open file pointsFile\n");
      }

    } else {

       oops("\ngenOutput: Can't get value of pointsFile\n");

    }
      if (velFile) {

        fp_vel = fopen(velFile, "w");

          if (!fp_vel) {
            oops("\ngenOutput: Can't open velFile\n");
          }

      } else {

         oops("\ngenOutput: Can't get value of velFile\n");

      }

      coords = (float *) malloc(3 * prod_dims * sizeof(float));
      vels = (float *) malloc(3 * prod_dims * sizeof(float));

      fprintf(fp_vel, "%d %d %d\n", dim1, dim2, dim3);
      fprintf(fp_points, "%d %d %d\n", dim1, dim2, dim3);
      for (i = 0; i < prod_dims; i++) {
         fscanf(fp_in, " %f %f %f ",
                &coords[3*i], &coords[3*i+1], &coords[3*i+2]);
         fscanf(fp_in, " %f %f %f ",
                &vels[3*i], &vels[3*i+1], &vels[3*i+2]);

         fprintf(fp_vel, "%e %e %e\n", vels[3*i], vels[3*i+1],
                 vels[3*i+2]);
      }

      /* Set velFile's value */
      OMset_name_str_val(ParssimParser_id, OMstr_to_name("velFile"),
                         velFile);

      for (i = 0; i < dim1; i++)
         fprintf(fp_points, "%f 0 0\n", coords[3*i]);
      for (i = 0; i < dim2; i++)
         fprintf(fp_points, "0 %f 0\n", coords[(3*i)*dim1 + 1]);
      for (i = 0; i < dim3; i++)
         fprintf(fp_points, "0 0 %f\n", coords[3*i*dim1*dim2 + 2]);

      /* Set pointsFile's value */
      OMset_name_str_val(ParssimParser_id, 
    			       OMstr_to_name("pointsFile"), pointsFile);

      free(coords);
      free(vels);
	fclose(fp_vel);
	fclose(fp_points);

  } else if (!strcmp(title, "pressure")) {

    presTime = prevTime; 
    OMset_name_int_val(ParssimParser_id, OMstr_to_name("presTime"),
                       presTime);
    strcpy(presFile, dataFile);
    if (pointsFile != NULL) {

      fp_points = fopen(pointsFile, "w");

      if (fp_points == NULL) {
        oops("\ngenOutput: Can't open pointsFile\n");
      }

    } else {

       oops("\ngenOutput: Can't get value of pointsFile\n");

    }

      if (presFile) {

        fp_pres = fopen(presFile, "w");

          if (!fp_pres) {
            oops("\ngenOutput: Can't open presFile\n");
          }

      } else {

         oops("\ngenOutput: Can't get value of presFile\n");

      }

      coords = (float *) malloc(3 * prod_dims * sizeof(float));
      pressures = (float *) malloc(prod_dims * sizeof(float));

      fprintf(fp_pres, "%d %d %d\n", dim1, dim2, dim3);
      fprintf(fp_points, "%d %d %d\n", dim1, dim2, dim3);
      for (i = 0; i < prod_dims; i++) {
         fscanf(fp_in, " %f %f %f ",
                &coords[3*i], &coords[3*i+1], &coords[3*i+2]);
         fscanf(fp_in, " %f ", &pressures[i]);

         fprintf(fp_pres, "%e\n", pressures[i]);
      }

      /* Set presFile's value */
      OMset_name_str_val(ParssimParser_id, OMstr_to_name("presFile"),
                         presFile);

      for (i = 0; i < dim1; i++)
         fprintf(fp_points, "%f 0 0\n", coords[3*i]);
      for (i = 0; i < dim2; i++)
         fprintf(fp_points, "0 %f 0\n", coords[(3*i)*dim1 + 1]);
      for (i = 0; i < dim3; i++)
         fprintf(fp_points, "0 0 %f\n", coords[3*i*dim1*dim2 + 2]);

      /* Set pointsFile's value */
      OMset_name_str_val(ParssimParser_id,
                         OMstr_to_name("pointsFile"), pointsFile);

      free(coords);
      free(pressures);
      fclose(fp_pres);
	fclose(fp_points);


  } else {

	ERRMSG("No input .plt file found. Enter name in user interface\n");
	   return(0);

  }
  fclose(fp_in);

  fprintf(stderr, "Done.\n");

  return (1);
} /* end of genOutput definition */

int getTimes(char *filename, int *curStep, int *curTime,
               int *prevStep, int *prevTime)
{
  FILE *fp_in;
  char scratch[1024];
  int i, c, nl;
  long prev, cur;

  if (filename != NULL) {

    fp_in = fopen(filename, "r");

    if (fp_in == NULL) {

	oops("\ngetTimes: Can't open time file\n");

    }

  } else {

     oops("\ngetTimes: Can't get value of filename\n");

  }

  nl = 0;
  while ((c = fgetc(fp_in)) != EOF) {
    if (c == '\n')
      ++nl;
  }
  rewind(fp_in);
  if (nl < 2) {
    fclose(fp_in);
    return (0);
  }
  for(i = 0; i < nl - 2; i++)
    fgets(scratch, sizeof(scratch), fp_in);
  prev = ftell(fp_in);
  fgets(scratch, sizeof(scratch), fp_in);
  cur  = ftell(fp_in);
  fseek(fp_in, cur, SEEK_SET);
  fscanf(fp_in, "%d %d", curStep, curTime);
  fseek(fp_in, prev, SEEK_SET);
  fscanf(fp_in, "%d %d", prevStep, prevTime);
  fclose(fp_in);
  return(1);
}

